@@ -118,15 +118,13 @@ class Blocker(object):
 
     def on_data(self, chunk_type, data):
         logging.error(u"type:%s data:%s" % (chunk_type, data))
-        if chunk_type == MULTI_BULK_REPLY and data and data[0] in self.callbacks :
+        if chunk_type == MULTI_BULK_REPLY and data and data[0] in self.callbacks:
             key = data[0]
-            callback = self.callbacks[key]
+            callback, timeout = self.callbacks[key]
             callback(data[1])
             del self.callbacks[key]
 
-
-    def blpop(self, key, callback, timeout=60):
-        self.callbacks[key] = callback
+    def _send_block(self, key, callback, timeout):
         try:
             self.conn.write(parse_command("blpop", key, timeout))
         except Exception, e:
@@ -136,6 +134,20 @@ class Blocker(object):
             del self.callbacks[key]
 
 
+    def blpop(self, key, callback, timeout=1800):
+        if key in self.callbacks:
+            last_callback, last_update = self.callbacks
+            self.callbacks[key] = (callback, time.time())
+            if time.time()-last_update >= timeout:
+                self._send_block(key, callback, timeout)
+                return
+            else:
+                return
+        else:
+            self.callbacks[key] = (callback, time.time())
+            self._send_block(key, callback, timeout)
+
+
 class Subscriber(object):
     def __init__(self, host, port, db):
         self.conn = None
